Package: nloptr
Type: Package
Title: R Interface to NLopt
Version: 2.2.1
Authors@R: c(person("Jelmer", "Ypma", role = "aut",
                    email = "uctpjyy@ucl.ac.uk"),
             person(c("Steven", "G."), "Johnson", role = "aut",
                    comment = "author of the NLopt C library"),
             person("Aymeric", "Stamm", role = c("ctb", "cre"),
                    email = "aymeric.stamm@cnrs.fr",
                    comment = c(ORCID = "0000-0002-8725-3654")),
             person(c("Hans", "W."), "Borchers", role = "ctb",
                    email = "hwborchers@googlemail.com"),
             person("Dirk", "Eddelbuettel", role = "ctb",
                    email = "edd@debian.org"),
             person("Brian", "Ripley", role = "ctb",
                    comment = "build process on multiple OS"),
             person("Kurt", "Hornik", role = "ctb",
                    comment = "build process on multiple OS"),
             person("Julien", "Chiquet", role = "ctb"),
             person("Avraham", "Adler", role = "ctb", 
                    email = "Avraham.Adler@gmail.com",
                    comment = c(ORCID = "0000-0002-3039-0703")),
             person("Xiongtao", "Dai", role = "ctb"),
             person("Jeroen", "Ooms", role = "ctb",
                    email = "jeroen@berkeley.edu"),
             person("Tomas", "Kalibera", role = "ctb"),
             person("Mikael", "Jagan", role = "ctb"))
Description: 
    Solve optimization problems using an R interface to NLopt. NLopt is a 
    free/open-source library for nonlinear optimization, providing a common
    interface for a number of different free optimization routines available
    online as well as original implementations of various other algorithms.
    See <https://nlopt.readthedocs.io/en/latest/NLopt_Algorithms/> for more
    information on the available algorithms. Building from included sources 
    requires 'CMake'. On Linux and 'macOS', if a suitable system build of 
    NLopt (2.7.0 or later) is found, it is used; otherwise, it is built 
    from included sources via 'CMake'. On Windows, NLopt is obtained through 
    'rwinlib' for 'R <= 4.1.x' or grabbed from the appropriate toolchain for
    'R >= 4.2.0'.
License: LGPL (>= 3)
SystemRequirements: cmake (>= 3.2.0) which is used only on Linux or
        macOS systems when no system build of nlopt (>= 2.7.0) can be
        found.
Encoding: UTF-8
RoxygenNote: 7.3.2
Suggests: knitr, rmarkdown, covr, tinytest
VignetteBuilder: knitr
URL: https://github.com/astamm/nloptr, https://astamm.github.io/nloptr/
BugReports: https://github.com/astamm/nloptr/issues
NeedsCompilation: yes
UseLTO: yes
Packaged: 2025-03-16 10:28:42 UTC; stamm-a
Author: Jelmer Ypma [aut],
  Steven G. Johnson [aut] (author of the NLopt C library),
  Aymeric Stamm [ctb, cre] (<https://orcid.org/0000-0002-8725-3654>),
  Hans W. Borchers [ctb],
  Dirk Eddelbuettel [ctb],
  Brian Ripley [ctb] (build process on multiple OS),
  Kurt Hornik [ctb] (build process on multiple OS),
  Julien Chiquet [ctb],
  Avraham Adler [ctb] (<https://orcid.org/0000-0002-3039-0703>),
  Xiongtao Dai [ctb],
  Jeroen Ooms [ctb],
  Tomas Kalibera [ctb],
  Mikael Jagan [ctb]
Maintainer: Aymeric Stamm <aymeric.stamm@cnrs.fr>
Repository: CRAN
Date/Publication: 2025-03-17 01:40:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 15:42:27 UTC; unix
Archs: nloptr.so.dSYM
